﻿package rtf;

import java.util.HashMap;

/**
 * Converts HTML entities by means of a dictionary-like hashmap
 * that holds the entities and the characters they represent.
 * 
 * @autor mtordesillas, hqcasanova
 * 
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/.
 */
public class HTMLDictionary {

	private HashMap<String, String> htmlDictionary;
	
	/**
	 * Default constructor. Populates the new dictionary with entries specified
	 * using a regular expression. This regex should take into account the two
	 * forms of entity notation: alphabetic and numeric.  
	 * The list of entities has been obtained from: http://www.w3schools.com/tags/ref_entities.asp
	 */
	public HTMLDictionary(){
		htmlDictionary = new HashMap<String, String>();
		
		htmlDictionary.put("&quot;|&#34;", "\"");
		htmlDictionary.put("&apos;|&#39;", "'");
		htmlDictionary.put("&amp;|&#38;", "&");
		htmlDictionary.put("&lt;|&#60;", "<");
		htmlDictionary.put("&gt;|&#62;", ">");
		htmlDictionary.put("&nbsp;|&#160;", " ");
		htmlDictionary.put("&iexcl;|&#161;", "¡");
		htmlDictionary.put("&cent;|&#162;", "¢");
		htmlDictionary.put("&pound;|&#163;", "£");
		htmlDictionary.put("&curren;|&#164;", "¤");
		htmlDictionary.put("&yen;|&#165;", "¥");
		htmlDictionary.put("&brvbar;|&#166;", "¦");
		htmlDictionary.put("&sect;|&#167;", "§");
		htmlDictionary.put("&uml;|&#168;", "¨");
		htmlDictionary.put("&copy;|&#169;", "©");
		htmlDictionary.put("&ordf;|&#170;", "ª");
		htmlDictionary.put("&laquo;|&#171;", "«");
		htmlDictionary.put("&not;|&#172;", "¬");
		htmlDictionary.put("&shy;|&#173;", " ");
		htmlDictionary.put("&reg;|&#174;", "®");
		htmlDictionary.put("&macr;|&#175;", "¯");
		htmlDictionary.put("&deg;|&#176;", "°");
		htmlDictionary.put("&plusmn;|&#177;", "±");
		htmlDictionary.put("&sup2;|&#178;", "²");
		htmlDictionary.put("&sup3;|&#179;", "³");
		htmlDictionary.put("&acute;|&#180;", "´");
		htmlDictionary.put("&micro;|&#181;", "µ");
		htmlDictionary.put("&para;|&#182;", "¶");
		htmlDictionary.put("&middot;|&#183;", "·");
		htmlDictionary.put("&cedil;|&#184;", "¸");
		htmlDictionary.put("&sup1;|&#185;", "¹");
		htmlDictionary.put("&ordm;|&#186;", "º");
		htmlDictionary.put("&raquo;|&#187;", "»");
		htmlDictionary.put("&frac14;|&#188;", "¼");
		htmlDictionary.put("&frac12;|&#189;", "½");
		htmlDictionary.put("&frac34;|&#190;", "¾");
		htmlDictionary.put("&iquest;|&#191;", "¿");
		htmlDictionary.put("&Agrave;|&#192;", "À");
		htmlDictionary.put("&Aacute;|&#193;", "Á");
		htmlDictionary.put("&Acirc;|&#194;", "Â");
		htmlDictionary.put("&Atilde;|&#195;", "Ã");
		htmlDictionary.put("&Auml;|&#196;", "Ä");
		htmlDictionary.put("&Aring;|&#197;", "Å");
		htmlDictionary.put("&AElig;|&#198;", "Æ");
		htmlDictionary.put("&Ccedil;|&#199;", "Ç");
		htmlDictionary.put("&Egrave;|&#200;", "È");
		htmlDictionary.put("&Eacute;|&#201;", "É");
		htmlDictionary.put("&Ecirc;|&#202;", "Ê");
		htmlDictionary.put("&Euml;|&#203;", "Ë");
		htmlDictionary.put("&Igrave;|&#204;", "Ì");
		htmlDictionary.put("&Iacute;|&#205;", "Í");
		htmlDictionary.put("&Icirc;|&#206;", "Î");
		htmlDictionary.put("&Iuml;|&#207;", "Ï");
		htmlDictionary.put("&ETH;|&#208;", "Ð");
		htmlDictionary.put("&Ntilde;|&#209;", "Ñ");
		htmlDictionary.put("&Ograve;|&#210;", "Ò");
		htmlDictionary.put("&Oacute;|&#211;", "Ó");
		htmlDictionary.put("&Ocirc;|&#212;", "Ô");
		htmlDictionary.put("&Otilde;|&#213;", "Õ");
		htmlDictionary.put("&Ouml;|&#214;", "Ö");
		htmlDictionary.put("&times;|&#215;", "×");
		htmlDictionary.put("&Oslash;|&#216;", "Ø");
		htmlDictionary.put("&Ugrave;|&#217;", "Ù");
		htmlDictionary.put("&Uacute;|&#218;", "Ú");
		htmlDictionary.put("&Ucirc;|&#219;", "Û");
		htmlDictionary.put("&Uuml;|&#220;", "Ü");
		htmlDictionary.put("&Yacute;|&#221;", "Ý");
		htmlDictionary.put("&THORN;|&#222;", "Þ");
		htmlDictionary.put("&szlig;|&#223;", "ß");
		htmlDictionary.put("&agrave;|&#224;", "à");
		htmlDictionary.put("&aacute;|&#225;", "á");
		htmlDictionary.put("&acirc;|&#226;", "â");
		htmlDictionary.put("&atilde;|&#227;", "ã");
		htmlDictionary.put("&auml;|&#228;", "ä");
		htmlDictionary.put("&aring;|&#229;", "å");
		htmlDictionary.put("&aelig;|&#230;", "æ");
		htmlDictionary.put("&ccedil;|&#231;", "ç");
		htmlDictionary.put("&egrave;|&#232;", "è");
		htmlDictionary.put("&eacute;|&#233;", "é");
		htmlDictionary.put("&ecirc;|&#234;", "ê");
		htmlDictionary.put("&euml;|&#235;", "ë");
		htmlDictionary.put("&igrave;|&#236;", "ì");
		htmlDictionary.put("&iacute;|&#237;", "í");
		htmlDictionary.put("&icirc;|&#238;", "î");
		htmlDictionary.put("&iuml;|&#239;", "ï");
		htmlDictionary.put("&eth;|&#240;", "ð");
		htmlDictionary.put("&ntilde;|&#241;", "ñ");
		htmlDictionary.put("&ograve;|&#242;", "ò");
		htmlDictionary.put("&oacute;|&#243;", "ó");
		htmlDictionary.put("&ocirc;|&#244;", "ô");
		htmlDictionary.put("&otilde;|&#245;", "õ");
		htmlDictionary.put("&ouml;|&#246;", "ö");
		htmlDictionary.put("&divide|&#247;", "÷");
		htmlDictionary.put("&oslash;|&#248;", "ø");
		htmlDictionary.put("&ugrave;|&#249;", "ù");
		htmlDictionary.put("&uacute;|&#250;", "ú");
		htmlDictionary.put("&ucirc;|&#251;", "û");
		htmlDictionary.put("&uuml;|&#252;", "ü");
		htmlDictionary.put("&yacute;|&#253;", "ý");
		htmlDictionary.put("&thorn;|&#254;", "þ");
		htmlDictionary.put("&yuml;|&#255;", "ÿ");		
	}
	
	/**
	 * Turns all HTML entities present in a given section of HTML code into their respective
	 * equivalent character.
	 * @param htmlString HTML code with the entities to translate
	 * @return HTML code with entities translated into the character they represent
	 */
	public String cleanHTMLCode(String htmlString){
		String tempHtml = htmlString;
		for (String key : htmlDictionary.keySet()){
			tempHtml = tempHtml.replaceAll (key, htmlDictionary.get(key));
		}
		return tempHtml;
	}	
}
