package utils;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GraphicsEnvironment;
import java.awt.Insets;
import java.awt.Toolkit;
import java.io.File;
import java.util.HashMap;
import java.util.Locale;
import java.util.ResourceBundle;

import editor.ConnectDB;

/**
 * Application-wide volatile constants
 * @author hqcasanova, jwdd, njerez
 * 
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/.
 */
public class Global
{
	//Editor's version number
	public final static String VERSION = "1.2.8";
	
	//System-dependent path separator
	public final static String SEPARADOR_SYS = System.getProperty ("file.separator");
	
	//HTML translation of one tab 
	public final static String TAB = "\u00A0\u00A0\u00A0\u00A0\u00A0\u00A0\u00A0\u00A0";
	
	//Fonts selected for display
	public final static String[] FONTS_DISPLAYABLE = { "Arial", "Arial Black", "Comic Sans MS", // fuentes por defecto
			"Courier New", "Georgia", "Impact", "Sarakanda", "Times New Roman", "Trebuchet MS", "Verdana" };
	
	//Maximum character number for document names
	public final static int DOCNAME_LENGTH = 30;
	
	//Maximum character number for virtual keypad buttons
	public final static int BUTTONNAME_LENGTH = 30;
	
	//Regular expression for alphabetic international characters
	public final static String ALPHA_REGEX = "a-zA-ZÂÃÄÀÁÅÆÇÈÉÊËÌÍÎÏÐÑÒÓÔÕÖØÙÚÛÜÝÞßàáâãäåæçèéêëìíîïðñòóôõöøùúûüýþÿ";

	//Regular expression for button names
	public final static String BUTTONNAME_REGEX = "^([" + ALPHA_REGEX + "0-9_\\-\\s]{1," + String.valueOf(BUTTONNAME_LENGTH) + "})$";
	
	//Regular expression for document names (numbers/dashes/spaces not allowed at the beginning of the name)
	public final static String DOCNAME_REGEX = "^([" + ALPHA_REGEX + "]|[" + ALPHA_REGEX + "][" + ALPHA_REGEX + "0-9_\\-\\s]{1," + String.valueOf(DOCNAME_LENGTH - 1) + "})$";

	//Regular expression for number of rows and columns (0-20) 
	public final static String TABLE_REGEX = "^(0?[1-9]|1[0-9]|20)$";
	
	//Data token separator
	public final static String TOKEN_SEPARATOR = "/";
	
	//Database handler (in charge of connecting, channeling queries and disconnecting)
	public static ConnectDB DB_HANDLER;
	
	//Database file name
	public static String DB_FILE = "";
	
	//Database file extension
	public final static String DB_EXT = "sqlite";
	
	//File extension for user documents
	public final static String DOC_EXT = "tca";
	
	//File extension for exported documents
	public final static String DOC_EXPORT_EXT = "rtf";
		
	//User configuration export file extension
	public final static String CONF_EXT = "cfg";
	
	//Pictogram folder
	public final static String PICT_FOLDER = "pict";
	
	//Pictogram file extension
	public final static String PICT_EXT = "png";
	
	//Pictogram file extension with dot
	public final static String PICT_EXT_PUNTO = "." +PICT_EXT;
	
	//Default pictogram id for new documents that originally had no pictogram associated
	public final static String DEFAULT_PICT = "0";
	
	//Fallback image file in case of IO errors.
	public final static String FILE_ERROR_IMAGE = PICT_FOLDER + SEPARADOR_SYS + DEFAULT_PICT + PICT_EXT_PUNTO;
		
	//Folder for image resources (e.g. button icons)
	public final static String IMG_FOLDER = "img";
	
	//Folder for all language-dependent databases
	public final static String DB_FOLDER = "db";
	
	//Installation directory application runs in 
	public final static String INSTALL_FOLDER = System.getProperty("user.dir");
	
	//Directory for role description files
	public final static String ROL_FOLDER = "role";
		
	//User profile root folder
	public final static String USERS_FOLDER = "profile";
	
	//Target folder for exported documents
	public final static String DOC_FOLDER = "docs";
		
	//System temp folder
	public final static String TEMP_FOLDER = System.getProperty("java.io.tmpdir");
	
	//Name for splash image
    public final static String SPLASH_IMG = "splash";
    
    //Name for splash credits image
    public final static String SPLASH_CREDITS = "splash2";
	
	//Name for print button icon
	public final static String PRINT = "guardar";
	
	//Name for credits button icon file
	public final static String CREDITS_IMG = "logo";
	
	//Name for avatar button icon
	public final static String USER_AVATAR = "user";
	
	//Name for user switch button icon
	public final static String USER_SWITCH = "cambiaUsr";
	
	//Name for new document button icon
	public final static String NEWDOC = "nuevoDocumento";
	
	//Name for load document button icon
	public final static String OPENDOC = "abrirDocumento";
	
	//Name for admin password dialogue icon
	public final static String ADMINLOCK = "candado";
	
	//Name for language selection dialogue icon
	public final static String LANGUAGESET = "idioma";
	
	//Name for intro banner image
	public final static String BANNER = "intro";
	
	//Editor's favicon (full path)
	public final static String FAVICON_ED = Global.IMG_FOLDER + Global.SEPARADOR_SYS + "favicon.png";
	
	//Minimised version for editor's favicon
	public final static String FAVICON_ED_MIN = Global.IMG_FOLDER + Global.SEPARADOR_SYS + "faviconMin.png";
	
	//Virtual keypad editor's favicon 
	public final static String FAVICON_TV = Global.IMG_FOLDER + Global.SEPARADOR_SYS + "tareas.png";
	
	//Button list for "editing" group
	public final static String G1_BUTTONS = "copiarOPegar" + TOKEN_SEPARATOR + "deshacer";
	
	//Button list for "font" group
	public final static String G2_BUTTONS = "fuente" + TOKEN_SEPARATOR + "color" + TOKEN_SEPARATOR + "aumentar" + TOKEN_SEPARATOR + "disminuir" + TOKEN_SEPARATOR + "negrita" + TOKEN_SEPARATOR + "cursiva" + TOKEN_SEPARATOR + "subrayado";
	
	//Button list for "insert" group
	public final static String G3_BUTTONS = "insertarImagen" + TOKEN_SEPARATOR + "insertarTabla" + TOKEN_SEPARATOR + "insertarVineta" + TOKEN_SEPARATOR + "insertarNumeracion";
	
	//Button list for "paragraph" group
	public final static String G4_BUTTONS = "sangria" + TOKEN_SEPARATOR + "interlineado";
	
	//Button list for "features" group (
	public final static String G5_BUTTONS = "pictogramas" + TOKEN_SEPARATOR + "tareas" + TOKEN_SEPARATOR + "cambiarColorFondo" + TOKEN_SEPARATOR + "resalta";
	
	//Highlight color for correct words in type 2 virtual keypads 
	public final static Color RIGHT = Color.decode("#0000FF");
	
	//Highlight color for incorrect words in type 2 virtual keypads
	public final static Color WRONG = Color.decode("#BF0000");
	
	//Highlight color for incorrect text fields
	public final static Color FIELD_ERROR_COLOR = Color.decode("#F28888");
	
	//Threshold between similar and different colors  
	public final static double COLOR_THRESHOLD = 17000.0;
	
	//Default line spacing
	public final static float LINE_SPACING = (float) 17.5;
	
	//Default indentation
	public final static float INDENTATION = (float) 70.0;
	
    //Default font type
	public final static String FONT_FAMILY = "Sarakanda";
	
	//Default font size (14 desktop publishing points) in pixels
	public final static int FONT_SIZE = 14; //Math.round ((14.0f / 72.0f) * Toolkit.getDefaultToolkit().getScreenResolution());
	
	//Current system's dot per inch value
	public final static int DPI = Toolkit.getDefaultToolkit().getScreenResolution();
	
	//Default padding (Word standard 1.27cm) in inches
	public final static float PADDING_INCH = 0.5f;
	
	//Default padding (Word standard 1.27cm) in pixels
	public final static int PADDING_PIX = Math.round (DPI * PADDING_INCH); 
	
	//Default page width (DIN A4) in pixels
	public final static int PAGE_WIDTH = Math.round (8.3f * DPI);
	
	//Default page height (DIN A4) in pixels
	public final static int PAGE_HEIGHT = Math.round (11.7f * DPI);
	
	//Default style for outer table borders
	public final static String TABLE_BORDER = "border-width: 2px; border-style: solid; border-color: black";
	
	//Default style for inner table borders
	public final static String TD_BORDER = "border-width: 1px; border-style: solid; border-color: black";
	
	//Maximum number of displayable users in dialogues (such as login)
	public final static int NMAXPAGUSER = 9;
	
	//Frequency for autosave (ms)
	public final static int AUTOSAVE_INTERVAL = 60000;
	
	//Time length before displaying previsualisation (ms)
	public final static int PREVI_RESPONSE = 2000;
	
	//Time duration of the previsualisation (ms)
	public final static int PREVI_INTERVAL = 4000;
	
	//Minimum response time for dynamic text fields (those triggering database query on button press)
	public final static int FIELD_RESPONSE = 10;
	
	//Tooltip background color
	public final static Color TOOLTIP_BACKGROUND = Color.decode ("#ADD9FF");
	
	//Default button background
	public final static Color BUTTON_BACKGROUND = Color.decode ("#FFFFFF");
	
	//Default border color
	public final static Color DEFAULT_BORDER = Color.decode("#C2E2FF");
	
	//Default highlight color for text selection
	public final static Color DEFAULT_HIGHLIGHT = Color.decode("#409BEB");
	
	//Default background color for panels
	public final static Color PANEL_BACKGROUND = Color.decode("#EEEEEE");
	
	//Graphics infrastructure object
	public final static GraphicsEnvironment GRAPHICS = GraphicsEnvironment.getLocalGraphicsEnvironment();
	
	//Current screen dimensions in pixels
	public final static Dimension SCREEN_DIM = Toolkit.getDefaultToolkit().getScreenSize();
	
    //Size of swatch in colour selection dialogue
	public final static int SWATCH_SIZE = Math.round (Global.SCREEN_DIM.height / 42.67f);
	
	//Default icon size in pixels
	public final static int ICON_SIZE = Math.round (Global.SCREEN_DIM.height / 22.22f);
	
	//Icon border width in pixels
	public final static int BUTTON_EDGE = Math.round ((float) SCREEN_DIM.getWidth() / 410.0f);
	
	//Default dimensions of button margins
	public final static Insets BUTTON_MARGINS = new Insets (Global.BUTTON_EDGE, Global.BUTTON_EDGE, Global.BUTTON_EDGE, Global.BUTTON_EDGE); 
	
	//Width for information-only dialogues in pixels
	public final static int FRAME_WIDTH = Math.round ((float) SCREEN_DIM.getWidth() / 2.55f);
	
	//Height for information-only dialogues in pixels
	public final static int FRAME_HEIGHT = Math.round ((float) SCREEN_DIM.getHeight() / 2.34f);
	
	//Width for splash screen
	public final static int SPLASH_WIDTH = Math.round ((float) SCREEN_DIM.getHeight() / 5.12f);
	
	//Default font for interface components
	public final static Font INTERFACE_FONT = new Font("Trebuchet MS", Font.PLAIN, Math.round ((float) SCREEN_DIM.getHeight() / 63.75f));
	
	//Name for user's button configuration XML file 
	public final static String USER_CONF_XML = "conf.xml";
	
	//XML schema file for user's button configuration file  
	public final static String USER_CONF_XSD = "conf.xsd";
	
	//Text file with current user's name
	public final static String USER_FILE_NAME = "usuario";
	
	//Text file listing all user's documents to be imported
	public final static String USER_IMPORT_DOC_LIST = "documentos";
	
	//Folder with all the user's default documents
	public final static String USER_DEFAULT_DATA_FOLDER = "default";
	
	//Temporary id for any user's profile being copied
	public final static int USER_DEFAULT_TEMP_ID = 0;
	
	//Temporary folder used during the pictogram update process
	public final static String UPDATE_FOLDER = "tmp_update";
	
	//Temporary folder with the updated pictograms
	public final static String UPDATE_FOLDER_PICT = "picto";
	
	//Name for the SQL scripts for inserting pictograms.
	public final static String UPDATE_INSERTSQL = "insert.sql";
	
	//Folder for all type 1 virtual keypad configuration files
	public final static String TCV1_FOLDER = "tcv1";
	
	//Folder for all type 2 virtual keypad configuration files
	public final static String TCV2_FOLDER = "tcv2";
	
	//Extension for virtual keypad configuration files
	public final static String TCV_EXT = "xml";
	
	//Types of virtual keypads.
	public final static int TIPO_TCV1 = 1;
	public final static int TIPO_TCV2 = 2;
	
	//Type 2 virtual keypad images folder	
	public final static String TCV2_IMG_FOLDER = "tcv2" + File.separator + IMG_FOLDER;
		
	//Full path for the color button icon
	public final static String RUTA_COLOR = IMG_FOLDER + File.separator + "color.png";
	
	//Full path for the banner at the top of virtual keypad editor's first window 
	public final static String RUTA_ADAPRO_TCV = "img" + File.separator + "protad.png";
	
	//Full path for the icon representing type 1 virtual keypads
	public final static String RUTA_IMG_TCV1 = "img" + File.separator + "tv1.png";
	
	//Full path for the icon representing type 2 virtual keypads
	public final static String RUTA_IMG_TCV2 = "img" + File.separator + "tv2.png";

	//i18n
	@SuppressWarnings("serial")
	public final static HashMap<String, Locale> LANGUAGES = new HashMap<String, Locale>()
	{{
			put("English", new Locale("en", "GB"));
			put("Español", new Locale("es", "ES"));
			put("Português", new Locale("pt", "PT"));
	}};
	public static ResourceBundle LANG = ResourceBundle.getBundle("MessagesBundle", Locale.getDefault(), Thread.currentThread().getContextClassLoader());

	//Default style-related settings for the editor's administrative interface
	public final static Font FONT_ADMIN_TITLE = new Font(Global.INTERFACE_FONT.getFontName(), Font.PLAIN, 26);
	public final static Font FONT_ADMIN_USER = new Font(Global.INTERFACE_FONT.getFontName(), Font.PLAIN, 16);
	public final static Font FONT_ADMIN_BUTTON = new Font(Global.INTERFACE_FONT.getFontName(), Font.PLAIN, 18);
	public final static Font FONT_TCV_TFIELD = new Font(Global.INTERFACE_FONT.getFontName(), Font.PLAIN, 16);
	public final static Color COLOR_FONDO = new Color(0, 85, 173);// (213,226,237); //(71, 159, 237); // 0 126 237 / 71 159 237 /118 181 237 / 165 204 237
	public final static Color COLOR_TEXTO = new Color(43, 43, 43); // 42, 86, 86);
	public final static Color COLOR_TEXTO_BOTON = new Color(43, 43, 43); // 31, 63, 63);
	public final static Color COLOR_CONTENIDO = new Color(255, 255, 255);// (245, 245, 245);
	
	
}
